# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_aux_input_decoration import VehicleStatsAuxInputDecoration
from .vehicle_stats_decorations_ambient_air_temperature_milli_c import (
    VehicleStatsDecorationsAmbientAirTemperatureMilliC,
)
from .vehicle_stats_decorations_barometric_pressure_pa import VehicleStatsDecorationsBarometricPressurePa
from .vehicle_stats_decorations_battery_milli_volts import VehicleStatsDecorationsBatteryMilliVolts
from .vehicle_stats_decorations_def_level_milli_percent import VehicleStatsDecorationsDefLevelMilliPercent
from .vehicle_stats_decorations_ecu_speed_mph import VehicleStatsDecorationsEcuSpeedMph
from .vehicle_stats_decorations_engine_coolant_temperature_milli_c import (
    VehicleStatsDecorationsEngineCoolantTemperatureMilliC,
)
from .vehicle_stats_decorations_engine_load_percent import VehicleStatsDecorationsEngineLoadPercent
from .vehicle_stats_decorations_engine_oil_pressure_k_pa import VehicleStatsDecorationsEngineOilPressureKPa
from .vehicle_stats_decorations_engine_rpm import VehicleStatsDecorationsEngineRpm
from .vehicle_stats_decorations_engine_states import VehicleStatsDecorationsEngineStates
from .vehicle_stats_decorations_fuel_consumed_milliliters import VehicleStatsDecorationsFuelConsumedMilliliters
from .vehicle_stats_decorations_fuel_percents import VehicleStatsDecorationsFuelPercents
from .vehicle_stats_decorations_gps import VehicleStatsDecorationsGps
from .vehicle_stats_decorations_gps_distance_meters import VehicleStatsDecorationsGpsDistanceMeters
from .vehicle_stats_decorations_gps_odometer_meters import VehicleStatsDecorationsGpsOdometerMeters
from .vehicle_stats_decorations_idling_duration_milliseconds import VehicleStatsDecorationsIdlingDurationMilliseconds
from .vehicle_stats_decorations_intake_manifold_temperature_milli_c import (
    VehicleStatsDecorationsIntakeManifoldTemperatureMilliC,
)
from .vehicle_stats_decorations_obd_engine_seconds import VehicleStatsDecorationsObdEngineSeconds
from .vehicle_stats_decorations_obd_odometer_meters import VehicleStatsDecorationsObdOdometerMeters
from .vehicle_stats_ecu_door_status import VehicleStatsEcuDoorStatus
from .vehicle_stats_engine_immobilizer import VehicleStatsEngineImmobilizer
from .vehicle_stats_ev_average_battery_temperature_milli_celsius import (
    VehicleStatsEvAverageBatteryTemperatureMilliCelsius,
)
from .vehicle_stats_ev_battery_current_milli_amp import VehicleStatsEvBatteryCurrentMilliAmp
from .vehicle_stats_ev_battery_state_of_health_milli_percent import VehicleStatsEvBatteryStateOfHealthMilliPercent
from .vehicle_stats_ev_battery_voltage_milli_volt import VehicleStatsEvBatteryVoltageMilliVolt
from .vehicle_stats_ev_charging_current_milli_amp import VehicleStatsEvChargingCurrentMilliAmp
from .vehicle_stats_ev_charging_energy_micro_wh import VehicleStatsEvChargingEnergyMicroWh
from .vehicle_stats_ev_charging_status import VehicleStatsEvChargingStatus
from .vehicle_stats_ev_charging_voltage_milli_volt import VehicleStatsEvChargingVoltageMilliVolt
from .vehicle_stats_ev_consumed_energy_micro_wh import VehicleStatsEvConsumedEnergyMicroWh
from .vehicle_stats_ev_distance_driven_meters import VehicleStatsEvDistanceDrivenMeters
from .vehicle_stats_ev_regenerated_energy_micro_wh import VehicleStatsEvRegeneratedEnergyMicroWh
from .vehicle_stats_ev_state_of_charge_milli_percent import VehicleStatsEvStateOfChargeMilliPercent
from .vehicle_stats_fault_codes_value import VehicleStatsFaultCodesValue
from .vehicle_stats_seatbelt_driver import VehicleStatsSeatbeltDriver
from .vehicle_stats_spreader_active import VehicleStatsSpreaderActive
from .vehicle_stats_spreader_air_temp import VehicleStatsSpreaderAirTemp
from .vehicle_stats_spreader_blast_state import VehicleStatsSpreaderBlastState
from .vehicle_stats_spreader_granular_name import VehicleStatsSpreaderGranularName
from .vehicle_stats_spreader_granular_rate import VehicleStatsSpreaderGranularRate
from .vehicle_stats_spreader_liquid_name import VehicleStatsSpreaderLiquidName
from .vehicle_stats_spreader_liquid_rate import VehicleStatsSpreaderLiquidRate
from .vehicle_stats_spreader_on_state import VehicleStatsSpreaderOnState
from .vehicle_stats_spreader_plow_status import VehicleStatsSpreaderPlowStatus
from .vehicle_stats_spreader_prewet_name import VehicleStatsSpreaderPrewetName
from .vehicle_stats_spreader_prewet_rate import VehicleStatsSpreaderPrewetRate
from .vehicle_stats_spreader_road_temp import VehicleStatsSpreaderRoadTemp
from .vehicle_stats_tire_pressures import VehicleStatsTirePressures


class VehicleStatsDecorations(UniversalBaseModel):
    """
    Optional decorations to the primary stat event. See [here](doc:decorations) for more details. The example shows the response if you were to submit `decorations=engineStates&obdEngineSeconds` to the query parameter:

    ```json
    "decorations":{
      "engineStates": {
        "value": "Off"
      },
      "obdEngineSeconds": {
        "value": 9723103
      }
    }
    ```
    """

    ambient_air_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsAmbientAirTemperatureMilliC],
        FieldMetadata(alias="ambientAirTemperatureMilliC"),
        pydantic.Field(alias="ambientAirTemperatureMilliC"),
    ] = None
    aux_input_1: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput1"),
        pydantic.Field(alias="auxInput1"),
    ] = None
    aux_input_10: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput10"),
        pydantic.Field(alias="auxInput10"),
    ] = None
    aux_input_11: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput11"),
        pydantic.Field(alias="auxInput11"),
    ] = None
    aux_input_12: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput12"),
        pydantic.Field(alias="auxInput12"),
    ] = None
    aux_input_13: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput13"),
        pydantic.Field(alias="auxInput13"),
    ] = None
    aux_input_2: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput2"),
        pydantic.Field(alias="auxInput2"),
    ] = None
    aux_input_3: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput3"),
        pydantic.Field(alias="auxInput3"),
    ] = None
    aux_input_4: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput4"),
        pydantic.Field(alias="auxInput4"),
    ] = None
    aux_input_5: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput5"),
        pydantic.Field(alias="auxInput5"),
    ] = None
    aux_input_6: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput6"),
        pydantic.Field(alias="auxInput6"),
    ] = None
    aux_input_7: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput7"),
        pydantic.Field(alias="auxInput7"),
    ] = None
    aux_input_8: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput8"),
        pydantic.Field(alias="auxInput8"),
    ] = None
    aux_input_9: typing_extensions.Annotated[
        typing.Optional[VehicleStatsAuxInputDecoration],
        FieldMetadata(alias="auxInput9"),
        pydantic.Field(alias="auxInput9"),
    ] = None
    barometric_pressure_pa: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsBarometricPressurePa],
        FieldMetadata(alias="barometricPressurePa"),
        pydantic.Field(alias="barometricPressurePa"),
    ] = None
    battery_milli_volts: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsBatteryMilliVolts],
        FieldMetadata(alias="batteryMilliVolts"),
        pydantic.Field(alias="batteryMilliVolts"),
    ] = None
    def_level_milli_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsDefLevelMilliPercent],
        FieldMetadata(alias="defLevelMilliPercent"),
        pydantic.Field(alias="defLevelMilliPercent"),
    ] = None
    ecu_door_status: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEcuDoorStatus],
        FieldMetadata(alias="ecuDoorStatus"),
        pydantic.Field(alias="ecuDoorStatus"),
    ] = None
    ecu_speed_mph: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsEcuSpeedMph],
        FieldMetadata(alias="ecuSpeedMph"),
        pydantic.Field(alias="ecuSpeedMph"),
    ] = None
    engine_coolant_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsEngineCoolantTemperatureMilliC],
        FieldMetadata(alias="engineCoolantTemperatureMilliC"),
        pydantic.Field(alias="engineCoolantTemperatureMilliC"),
    ] = None
    engine_immobilizer: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEngineImmobilizer],
        FieldMetadata(alias="engineImmobilizer"),
        pydantic.Field(alias="engineImmobilizer"),
    ] = None
    engine_load_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsEngineLoadPercent],
        FieldMetadata(alias="engineLoadPercent"),
        pydantic.Field(alias="engineLoadPercent"),
    ] = None
    engine_oil_pressure_k_pa: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsEngineOilPressureKPa],
        FieldMetadata(alias="engineOilPressureKPa"),
        pydantic.Field(alias="engineOilPressureKPa"),
    ] = None
    engine_rpm: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsEngineRpm],
        FieldMetadata(alias="engineRpm"),
        pydantic.Field(alias="engineRpm"),
    ] = None
    engine_states: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsEngineStates],
        FieldMetadata(alias="engineStates"),
        pydantic.Field(alias="engineStates"),
    ] = None
    ev_average_battery_temperature_milli_celsius: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvAverageBatteryTemperatureMilliCelsius],
        FieldMetadata(alias="evAverageBatteryTemperatureMilliCelsius"),
        pydantic.Field(alias="evAverageBatteryTemperatureMilliCelsius"),
    ] = None
    ev_battery_current_milli_amp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvBatteryCurrentMilliAmp],
        FieldMetadata(alias="evBatteryCurrentMilliAmp"),
        pydantic.Field(alias="evBatteryCurrentMilliAmp"),
    ] = None
    ev_battery_state_of_health_milli_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvBatteryStateOfHealthMilliPercent],
        FieldMetadata(alias="evBatteryStateOfHealthMilliPercent"),
        pydantic.Field(alias="evBatteryStateOfHealthMilliPercent"),
    ] = None
    ev_battery_voltage_milli_volt: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvBatteryVoltageMilliVolt],
        FieldMetadata(alias="evBatteryVoltageMilliVolt"),
        pydantic.Field(alias="evBatteryVoltageMilliVolt"),
    ] = None
    ev_charging_current_milli_amp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvChargingCurrentMilliAmp],
        FieldMetadata(alias="evChargingCurrentMilliAmp"),
        pydantic.Field(alias="evChargingCurrentMilliAmp"),
    ] = None
    ev_charging_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvChargingEnergyMicroWh],
        FieldMetadata(alias="evChargingEnergyMicroWh"),
        pydantic.Field(alias="evChargingEnergyMicroWh"),
    ] = None
    ev_charging_status: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvChargingStatus],
        FieldMetadata(alias="evChargingStatus"),
        pydantic.Field(alias="evChargingStatus"),
    ] = None
    ev_charging_voltage_milli_volt: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvChargingVoltageMilliVolt],
        FieldMetadata(alias="evChargingVoltageMilliVolt"),
        pydantic.Field(alias="evChargingVoltageMilliVolt"),
    ] = None
    ev_consumed_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvConsumedEnergyMicroWh],
        FieldMetadata(alias="evConsumedEnergyMicroWh"),
        pydantic.Field(alias="evConsumedEnergyMicroWh"),
    ] = None
    ev_distance_driven_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvDistanceDrivenMeters],
        FieldMetadata(alias="evDistanceDrivenMeters"),
        pydantic.Field(alias="evDistanceDrivenMeters"),
    ] = None
    ev_regenerated_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvRegeneratedEnergyMicroWh],
        FieldMetadata(alias="evRegeneratedEnergyMicroWh"),
        pydantic.Field(alias="evRegeneratedEnergyMicroWh"),
    ] = None
    ev_state_of_charge_milli_percent: typing_extensions.Annotated[
        typing.Optional[VehicleStatsEvStateOfChargeMilliPercent],
        FieldMetadata(alias="evStateOfChargeMilliPercent"),
        pydantic.Field(alias="evStateOfChargeMilliPercent"),
    ] = None
    fault_codes: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValue],
        FieldMetadata(alias="faultCodes"),
        pydantic.Field(alias="faultCodes"),
    ] = None
    fuel_consumed_milliliters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsFuelConsumedMilliliters],
        FieldMetadata(alias="fuelConsumedMilliliters"),
        pydantic.Field(alias="fuelConsumedMilliliters"),
    ] = None
    fuel_percents: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsFuelPercents],
        FieldMetadata(alias="fuelPercents"),
        pydantic.Field(alias="fuelPercents"),
    ] = None
    gps: typing.Optional[VehicleStatsDecorationsGps] = None
    gps_distance_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsGpsDistanceMeters],
        FieldMetadata(alias="gpsDistanceMeters"),
        pydantic.Field(alias="gpsDistanceMeters"),
    ] = None
    gps_odometer_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsGpsOdometerMeters],
        FieldMetadata(alias="gpsOdometerMeters"),
        pydantic.Field(alias="gpsOdometerMeters"),
    ] = None
    idling_duration_milliseconds: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsIdlingDurationMilliseconds],
        FieldMetadata(alias="idlingDurationMilliseconds"),
        pydantic.Field(alias="idlingDurationMilliseconds"),
    ] = None
    intake_manifold_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsIntakeManifoldTemperatureMilliC],
        FieldMetadata(alias="intakeManifoldTemperatureMilliC"),
        pydantic.Field(alias="intakeManifoldTemperatureMilliC"),
    ] = None
    obd_engine_seconds: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsObdEngineSeconds],
        FieldMetadata(alias="obdEngineSeconds"),
        pydantic.Field(alias="obdEngineSeconds"),
    ] = None
    obd_odometer_meters: typing_extensions.Annotated[
        typing.Optional[VehicleStatsDecorationsObdOdometerMeters],
        FieldMetadata(alias="obdOdometerMeters"),
        pydantic.Field(alias="obdOdometerMeters"),
    ] = None
    seatbelt_driver: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSeatbeltDriver],
        FieldMetadata(alias="seatbeltDriver"),
        pydantic.Field(alias="seatbeltDriver"),
    ] = None
    spreader_active: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderActive],
        FieldMetadata(alias="spreaderActive"),
        pydantic.Field(alias="spreaderActive"),
    ] = None
    spreader_air_temp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderAirTemp],
        FieldMetadata(alias="spreaderAirTemp"),
        pydantic.Field(alias="spreaderAirTemp"),
    ] = None
    spreader_blast_state: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderBlastState],
        FieldMetadata(alias="spreaderBlastState"),
        pydantic.Field(alias="spreaderBlastState"),
    ] = None
    spreader_granular_name: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderGranularName],
        FieldMetadata(alias="spreaderGranularName"),
        pydantic.Field(alias="spreaderGranularName"),
    ] = None
    spreader_granular_rate: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderGranularRate],
        FieldMetadata(alias="spreaderGranularRate"),
        pydantic.Field(alias="spreaderGranularRate"),
    ] = None
    spreader_liquid_name: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderLiquidName],
        FieldMetadata(alias="spreaderLiquidName"),
        pydantic.Field(alias="spreaderLiquidName"),
    ] = None
    spreader_liquid_rate: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderLiquidRate],
        FieldMetadata(alias="spreaderLiquidRate"),
        pydantic.Field(alias="spreaderLiquidRate"),
    ] = None
    spreader_on_state: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderOnState],
        FieldMetadata(alias="spreaderOnState"),
        pydantic.Field(alias="spreaderOnState"),
    ] = None
    spreader_plow_status: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderPlowStatus],
        FieldMetadata(alias="spreaderPlowStatus"),
        pydantic.Field(alias="spreaderPlowStatus"),
    ] = None
    spreader_prewet_name: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderPrewetName],
        FieldMetadata(alias="spreaderPrewetName"),
        pydantic.Field(alias="spreaderPrewetName"),
    ] = None
    spreader_prewet_rate: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderPrewetRate],
        FieldMetadata(alias="spreaderPrewetRate"),
        pydantic.Field(alias="spreaderPrewetRate"),
    ] = None
    spreader_road_temp: typing_extensions.Annotated[
        typing.Optional[VehicleStatsSpreaderRoadTemp],
        FieldMetadata(alias="spreaderRoadTemp"),
        pydantic.Field(alias="spreaderRoadTemp"),
    ] = None
    tire_pressure: typing_extensions.Annotated[
        typing.Optional[VehicleStatsTirePressures],
        FieldMetadata(alias="tirePressure"),
        pydantic.Field(alias="tirePressure"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.inline_response_2007 import InlineResponse2007
from ..types.v_1_trailer_assignments_response import V1TrailerAssignmentsResponse
from .raw_client import AsyncRawTrailerAssignmentsClient, RawTrailerAssignmentsClient


class TrailerAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTrailerAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTrailerAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTrailerAssignmentsClient
        """
        return self._raw_client

    def v_1_get_all_trailer_assignments(
        self,
        *,
        start_ms: typing.Optional[int] = None,
        end_ms: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        starting_after: typing.Optional[str] = None,
        ending_before: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InlineResponse2007:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch trailer assignment data for all trailers in your organization.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_ms : typing.Optional[int]
            Timestamp in Unix epoch miliseconds representing the start of the period to fetch. Omitting both startMs and endMs only returns current assignments.

        end_ms : typing.Optional[int]
            Timestamp in Unix epoch miliseconds representing the end of the period to fetch. Omitting endMs sets endMs as the current time

        limit : typing.Optional[int]
            Pagination parameter indicating the number of results to return in this request. Used in conjunction with either 'startingAfter' or 'endingBefore'.

        starting_after : typing.Optional[str]
            Pagination parameter indicating the cursor position to continue returning results after. Used in conjunction with the 'limit' parameter. Mutually exclusive with 'endingBefore' parameter.

        ending_before : typing.Optional[str]
            Pagination parameter indicating the cursor position to return results before. Used in conjunction with the 'limit' parameter. Mutually exclusive with 'startingAfter' parameter.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2007
            Returns trailer assignment data for all trailers in your organization

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.trailer_assignments.v_1_get_all_trailer_assignments()
        """
        _response = self._raw_client.v_1_get_all_trailer_assignments(
            start_ms=start_ms,
            end_ms=end_ms,
            limit=limit,
            starting_after=starting_after,
            ending_before=ending_before,
            request_options=request_options,
        )
        return _response.data

    def v_1_get_fleet_trailer_assignments(
        self,
        trailer_id: int,
        *,
        start_ms: typing.Optional[int] = None,
        end_ms: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> V1TrailerAssignmentsResponse:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch trailer assignment data for a single trailer.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        trailer_id : int
            ID of trailer. Must contain only digits 0-9.

        start_ms : typing.Optional[int]
            Timestamp in Unix epoch milliseconds representing the start of the period to fetch. Omitting both startMs and endMs only returns current assignments.

        end_ms : typing.Optional[int]
            Timestamp in Unix epoch milliseconds representing the end of the period to fetch. Omitting endMs sets endMs as the current time

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        V1TrailerAssignmentsResponse
            Returns trailer assignment data for a single trailer

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.trailer_assignments.v_1_get_fleet_trailer_assignments(
            trailer_id=1000000,
        )
        """
        _response = self._raw_client.v_1_get_fleet_trailer_assignments(
            trailer_id, start_ms=start_ms, end_ms=end_ms, request_options=request_options
        )
        return _response.data


class AsyncTrailerAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTrailerAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTrailerAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTrailerAssignmentsClient
        """
        return self._raw_client

    async def v_1_get_all_trailer_assignments(
        self,
        *,
        start_ms: typing.Optional[int] = None,
        end_ms: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        starting_after: typing.Optional[str] = None,
        ending_before: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InlineResponse2007:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch trailer assignment data for all trailers in your organization.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_ms : typing.Optional[int]
            Timestamp in Unix epoch miliseconds representing the start of the period to fetch. Omitting both startMs and endMs only returns current assignments.

        end_ms : typing.Optional[int]
            Timestamp in Unix epoch miliseconds representing the end of the period to fetch. Omitting endMs sets endMs as the current time

        limit : typing.Optional[int]
            Pagination parameter indicating the number of results to return in this request. Used in conjunction with either 'startingAfter' or 'endingBefore'.

        starting_after : typing.Optional[str]
            Pagination parameter indicating the cursor position to continue returning results after. Used in conjunction with the 'limit' parameter. Mutually exclusive with 'endingBefore' parameter.

        ending_before : typing.Optional[str]
            Pagination parameter indicating the cursor position to return results before. Used in conjunction with the 'limit' parameter. Mutually exclusive with 'startingAfter' parameter.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InlineResponse2007
            Returns trailer assignment data for all trailers in your organization

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.trailer_assignments.v_1_get_all_trailer_assignments()


        asyncio.run(main())
        """
        _response = await self._raw_client.v_1_get_all_trailer_assignments(
            start_ms=start_ms,
            end_ms=end_ms,
            limit=limit,
            starting_after=starting_after,
            ending_before=ending_before,
            request_options=request_options,
        )
        return _response.data

    async def v_1_get_fleet_trailer_assignments(
        self,
        trailer_id: int,
        *,
        start_ms: typing.Optional[int] = None,
        end_ms: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> V1TrailerAssignmentsResponse:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch trailer assignment data for a single trailer.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        trailer_id : int
            ID of trailer. Must contain only digits 0-9.

        start_ms : typing.Optional[int]
            Timestamp in Unix epoch milliseconds representing the start of the period to fetch. Omitting both startMs and endMs only returns current assignments.

        end_ms : typing.Optional[int]
            Timestamp in Unix epoch milliseconds representing the end of the period to fetch. Omitting endMs sets endMs as the current time

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        V1TrailerAssignmentsResponse
            Returns trailer assignment data for a single trailer

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.trailer_assignments.v_1_get_fleet_trailer_assignments(
                trailer_id=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.v_1_get_fleet_trailer_assignments(
            trailer_id, start_ms=start_ms, end_ms=end_ms, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .speed_trigger_details_object_response_body_operation import SpeedTriggerDetailsObjectResponseBodyOperation


class SpeedTriggerDetailsObjectResponseBody(UniversalBaseModel):
    """
    Details specific to Speed
    """

    min_duration_milliseconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="minDurationMilliseconds"),
        pydantic.Field(
            alias="minDurationMilliseconds",
            description="The number of milliseconds the trigger needs to stay active before alerting.",
        ),
    ]
    operation: SpeedTriggerDetailsObjectResponseBodyOperation = pydantic.Field()
    """
    How to evaluate the threshold.  Valid values: `GREATER`, `LESS`
    """

    speed_kilometers_per_hour: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="speedKilometersPerHour"),
        pydantic.Field(alias="speedKilometersPerHour", description="The speed threshold value."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DriverVehicleAssignmentV2ObjectResponseBodyAssignmentType = typing.Union[
    typing.Literal[
        "invalid",
        "unknown",
        "HOS",
        "idCard",
        "static",
        "faceId",
        "tachograph",
        "safetyManual",
        "RFID",
        "trailer",
        "external",
        "qrCode",
        "driverApp",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

FuelPurchasePostFuelPurchaseRequestBodyFuelGrade = typing.Union[
    typing.Literal["Unknown", "Regular", "Premium"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.trailer_assignments_create_driver_trailer_assignment_response_body import (
    TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody,
)
from ..types.trailer_assignments_get_driver_trailer_assignments_response_body import (
    TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody,
)
from ..types.trailer_assignments_update_driver_trailer_assignment_response_body import (
    TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody,
)
from .raw_client import AsyncRawDriverTrailerAssignmentsClient, RawDriverTrailerAssignmentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DriverTrailerAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDriverTrailerAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDriverTrailerAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDriverTrailerAssignmentsClient
        """
        return self._raw_client

    def get_driver_trailer_assignments(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody:
        """
        Get currently active driver-trailer assignments for driver.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_trailer_assignments.get_driver_trailer_assignments()
        """
        _response = self._raw_client.get_driver_trailer_assignments(
            driver_ids=driver_ids,
            after=after,
            include_external_ids=include_external_ids,
            request_options=request_options,
        )
        return _response.data

    def create_driver_trailer_assignment(
        self,
        *,
        driver_id: str,
        trailer_id: str,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody:
        """
        Create a new driver-trailer assignment

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        trailer_id : str
            ID of the trailer. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the trailer.

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. The time needs to be current or within the past 7 days. Defaults to now if not provided

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_trailer_assignments.create_driver_trailer_assignment(
            driver_id="494123",
            trailer_id="12345",
        )
        """
        _response = self._raw_client.create_driver_trailer_assignment(
            driver_id=driver_id, trailer_id=trailer_id, start_time=start_time, request_options=request_options
        )
        return _response.data

    def update_driver_trailer_assignment(
        self, *, id: str, end_time: str, request_options: typing.Optional[RequestOptions] = None
    ) -> TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody:
        """
        Update an existing driver-trailer assignment.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Samsara ID for the assignment.

        end_time : str
            The end time in RFC 3339 format. The end time should not be in the future

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_trailer_assignments.update_driver_trailer_assignment(
            id="id",
            end_time="2019-06-13T19:08:25Z",
        )
        """
        _response = self._raw_client.update_driver_trailer_assignment(
            id=id, end_time=end_time, request_options=request_options
        )
        return _response.data


class AsyncDriverTrailerAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDriverTrailerAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDriverTrailerAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDriverTrailerAssignmentsClient
        """
        return self._raw_client

    async def get_driver_trailer_assignments(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody:
        """
        Get currently active driver-trailer assignments for driver.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailerAssignmentsGetDriverTrailerAssignmentsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_trailer_assignments.get_driver_trailer_assignments()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_driver_trailer_assignments(
            driver_ids=driver_ids,
            after=after,
            include_external_ids=include_external_ids,
            request_options=request_options,
        )
        return _response.data

    async def create_driver_trailer_assignment(
        self,
        *,
        driver_id: str,
        trailer_id: str,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody:
        """
        Create a new driver-trailer assignment

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        trailer_id : str
            ID of the trailer. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the trailer.

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. The time needs to be current or within the past 7 days. Defaults to now if not provided

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailerAssignmentsCreateDriverTrailerAssignmentResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_trailer_assignments.create_driver_trailer_assignment(
                driver_id="494123",
                trailer_id="12345",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_driver_trailer_assignment(
            driver_id=driver_id, trailer_id=trailer_id, start_time=start_time, request_options=request_options
        )
        return _response.data

    async def update_driver_trailer_assignment(
        self, *, id: str, end_time: str, request_options: typing.Optional[RequestOptions] = None
    ) -> TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody:
        """
        Update an existing driver-trailer assignment.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Samsara ID for the assignment.

        end_time : str
            The end time in RFC 3339 format. The end time should not be in the future

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrailerAssignmentsUpdateDriverTrailerAssignmentResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_trailer_assignments.update_driver_trailer_assignment(
                id="id",
                end_time="2019-06-13T19:08:25Z",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_driver_trailer_assignment(
            id=id, end_time=end_time, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .vehicle_stats_fault_codes_j_1939_trouble_code import VehicleStatsFaultCodesJ1939TroubleCode

VehicleStatsFaultCodesJ1939TroubleCodes = typing.List[VehicleStatsFaultCodesJ1939TroubleCode]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_assignment_metadata_tiny_object_response_body import DriverAssignmentMetadataTinyObjectResponseBody
from .driver_vehicle_assignment_v_2_object_response_body_assignment_type import (
    DriverVehicleAssignmentV2ObjectResponseBodyAssignmentType,
)
from .goa_driver_tiny_response_response_body import GoaDriverTinyResponseResponseBody
from .goa_vehicle_tiny_response_response_body import GoaVehicleTinyResponseResponseBody


class DriverVehicleAssignmentV2ObjectResponseBody(UniversalBaseModel):
    """
    Object with driver assignment info and associated driver and vehicle info.
    """

    assigned_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assignedAtTime"),
        pydantic.Field(
            alias="assignedAtTime",
            description=" An assigned at time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ] = None
    assignment_type: typing_extensions.Annotated[
        typing.Optional[DriverVehicleAssignmentV2ObjectResponseBodyAssignmentType],
        FieldMetadata(alias="assignmentType"),
        pydantic.Field(
            alias="assignmentType",
            description="Name of the assigning source for the driver assignment record.  Valid values: `invalid`, `unknown`, `HOS`, `idCard`, `static`, `faceId`, `tachograph`, `safetyManual`, `RFID`, `trailer`, `external`, `qrCode`, `driverApp`",
        ),
    ] = None
    driver: GoaDriverTinyResponseResponseBody
    end_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime",
            description=" An end time in RFC 3339 format. Omitted if not applicable. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ] = None
    is_passenger: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isPassenger"),
        pydantic.Field(alias="isPassenger", description="Boolean indicating whether the driver is a passenger."),
    ]
    metadata: typing.Optional[DriverAssignmentMetadataTinyObjectResponseBody] = None
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime",
            description=" A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ]
    vehicle: GoaVehicleTinyResponseResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PatchAssetDataOutputsSingleResponseResponseBody(UniversalBaseModel):
    """
    A response that corresponds to an element in the original request body.
    """

    error_message: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="errorMessage"),
        pydantic.Field(alias="errorMessage", description="If the request failed, this displays the error message."),
    ] = None
    id: str = pydantic.Field()
    """
    The data output ID.
    """

    status_code: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="statusCode"),
        pydantic.Field(
            alias="statusCode",
            description="The status code of the request. 200 indicates the request succeeded for this data output. 500 indicates an internal server error.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleResponseResponseBodyAssetType = typing.Union[
    typing.Literal["uncategorized", "trailer", "equipment", "unpowered", "vehicle"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

SafetySettingsObjectResponseBodyDefaultVehicleType = typing.Union[
    typing.Literal["off", "automatic", "passengerCar", "lightTruck", "heavyDuty"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .equipment_locations_response_data import EquipmentLocationsResponseData
from .pagination_response import PaginationResponse


class EquipmentLocationsResponse(UniversalBaseModel):
    """
    The most recent equipment locations and pagination information
    """

    data: typing.List[EquipmentLocationsResponseData] = pydantic.Field()
    """
    List of the most recent locations for the specified units of equipment.
    """

    pagination: PaginationResponse

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .training_course_object_response_body import TrainingCourseObjectResponseBody
from .training_learner_object_response_body import TrainingLearnerObjectResponseBody


class TrainingAssignmentPostResponseObjectResponseBody(UniversalBaseModel):
    """
    Training Assignment response object.
    """

    course: TrainingCourseObjectResponseBody
    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime",
            description="Creation time of the training assignment. UTC timestamp in RFC 3339 format.",
        ),
    ]
    created_by_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdById"),
        pydantic.Field(alias="createdById", description="ID of the user who created the training assignment."),
    ]
    due_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="dueAtTime"),
        pydantic.Field(
            alias="dueAtTime",
            description="Time training assignment is due. UTC timestamp in RFC 3339 format. Returned when an assignment has a due date set by an admin.",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the training assignment.
    """

    is_existing_assignment: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isExistingAssignment"),
        pydantic.Field(
            alias="isExistingAssignment", description="Indicates whether the training assignment was already created."
        ),
    ]
    learner: TrainingLearnerObjectResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .dvir_2_form_submission_object_response_body import Dvir2FormSubmissionObjectResponseBody
from .dvir_2_related_device_object_response_body import Dvir2RelatedDeviceObjectResponseBody
from .dvir_2_submission_response_object_response_body_inspection_type import (
    Dvir2SubmissionResponseObjectResponseBodyInspectionType,
)


class Dvir2SubmissionResponseObjectResponseBody(UniversalBaseModel):
    """
    A DVIR description.
    """

    base_form_submission: typing_extensions.Annotated[
        typing.Optional[Dvir2FormSubmissionObjectResponseBody],
        FieldMetadata(alias="baseFormSubmission"),
        pydantic.Field(alias="baseFormSubmission"),
    ] = None
    inspection_type: typing_extensions.Annotated[
        Dvir2SubmissionResponseObjectResponseBodyInspectionType,
        FieldMetadata(alias="inspectionType"),
        pydantic.Field(
            alias="inspectionType",
            description="Inspection type of the DVIR.  Valid values: `preTrip`, `postTrip`, `mechanic`, `unset`",
        ),
    ]
    related_devices: typing_extensions.Annotated[
        typing.List[Dvir2RelatedDeviceObjectResponseBody],
        FieldMetadata(alias="relatedDevices"),
        pydantic.Field(
            alias="relatedDevices", description="List of devices (vehicles/trailers) associated with the DVIR."
        ),
    ]
    uuid_: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="uuid"),
        pydantic.Field(alias="uuid", description="The unique UUID of the DVIR2 submission"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleStatsFaultCodesValueObdiiDiagnosticTroubleCodesIgnitionType = typing.Union[
    typing.Literal["spark", "compression"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ConditionalFieldSectionObjectResponseBody(UniversalBaseModel):
    conditional_field_first_index: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="conditionalFieldFirstIndex"),
        pydantic.Field(
            alias="conditionalFieldFirstIndex",
            description="The index of the first conditional field associated with the triggeringFieldValue in the fieldTypes list.",
        ),
    ] = None
    conditional_field_last_index: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="conditionalFieldLastIndex"),
        pydantic.Field(
            alias="conditionalFieldLastIndex",
            description="The index of the last conditional field associated with the triggeringFieldValue in the fieldTypes list.",
        ),
    ] = None
    triggering_field_index: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="triggeringFieldIndex"),
        pydantic.Field(
            alias="triggeringFieldIndex",
            description="The index of the multiple choice field in the fieldTypes list that triggers one or more conditional fields.",
        ),
    ] = None
    triggering_field_value: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="triggeringFieldValue"),
        pydantic.Field(
            alias="triggeringFieldValue",
            description="The multiple choice option value that triggers the conditional fields.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .reports_dataset_field_response_body import ReportsDatasetFieldResponseBody


class ReportsDatasetResponseBody(UniversalBaseModel):
    """
    Information about a dataset.
    """

    display_name: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="displayName"),
        pydantic.Field(alias="displayName", description="Display name of the dataset."),
    ]
    fields: typing.List[ReportsDatasetFieldResponseBody] = pydantic.Field()
    """
    Fields within the dataset.
    """

    has_time_range_filter: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="hasTimeRangeFilter"),
        pydantic.Field(alias="hasTimeRangeFilter", description="Indicates if the dataset has a time range filter."),
    ]
    id: str = pydantic.Field()
    """
    Unique ID of the dataset.
    """

    time_range_limit_days: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="timeRangeLimitDays"),
        pydantic.Field(alias="timeRangeLimitDays", description="Time range limit for the dataset measured in days."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .dvir_signature_signatory_user import DvirSignatureSignatoryUser
from .dvir_signature_type import DvirSignatureType


class DvirSignature(UniversalBaseModel):
    """
    DVIR Signure.
    """

    signatory_user: typing_extensions.Annotated[
        typing.Optional[DvirSignatureSignatoryUser],
        FieldMetadata(alias="signatoryUser"),
        pydantic.Field(alias="signatoryUser"),
    ] = None
    signed_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="signedAtTime"),
        pydantic.Field(
            alias="signedAtTime",
            description="The time when the DVIR was signed. UTC timestamp in RFC 3339 format. Example: `2020-01-27T07:06:25Z`.",
        ),
    ] = None
    type: typing.Optional[DvirSignatureType] = pydantic.Field(default=None)
    """
    Whether the DVIR was submitted by a `driver` or `mechanic`. Valid values: `driver`, `mechanic`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SignatureValueObjectResponseBody(UniversalBaseModel):
    """
    The value of a signature field. Only present for signature fields.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Id of the signature field.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the signee for a signature field.
    """

    signed_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="signedAtTime"),
        pydantic.Field(alias="signedAtTime", description="Time the signature was captured in RFC 3339 format."),
    ] = None
    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Url of a signature field's PNG signature image.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyScoreSpeedingObjectResponseBodySpeedingType = typing.Union[
    typing.Literal["light", "moderate", "heavy", "severe", "maxSpeed", "unknown"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .tachograph_driver_file_list_wrapper import TachographDriverFileListWrapper

TachographDriverFileData = typing.List[TachographDriverFileListWrapper]



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionStatusResponseObjectResponseBodyHealthStatus = typing.Union[
    typing.Literal[
        "Status Not Set",
        "Connected",
        "Not Installed",
        "Power Source Off - Active Vehicle",
        "Power Source Off - Inactive Vehicle",
        "Weak Cellular Signal",
        "Requires Investigation",
        "Requires Charge",
        "Unsupported Product",
        "Low Battery. Replace Device.",
        "Low Vehicle Battery",
        "Unplugged",
        "Low Charging State",
        "Vehicle Off",
        "Weak GPS Signal",
        "Low Gateway Battery",
        "Low Gateway Battery (AG24)",
        "Low Gateway Battery (AG45)",
        "Low Gateway Battery (AG26)",
        "Low Gateway Battery (AG46)",
        "Low Gateway Battery (AG46-P)",
        "Temporarily Offline",
        "Prolonged Offline",
        "Recently Offline",
        "Replacement Required",
        "Status Unknown",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .detection_log_asset_object_response_body import DetectionLogAssetObjectResponseBody
from .detection_log_detection_object_response_body_trigger_detection_label import (
    DetectionLogDetectionObjectResponseBodyTriggerDetectionLabel,
)
from .detection_log_driver_object_response_body import DetectionLogDriverObjectResponseBody
from .detection_log_safety_event_object_response_body import DetectionLogSafetyEventObjectResponseBody


class DetectionLogDetectionObjectResponseBody(UniversalBaseModel):
    """
    Detection log entry
    """

    asset: DetectionLogAssetObjectResponseBody
    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime",
            description="Time the detection was detected or in-cab alert played in UTC. RFC 3339 format. Always returned.",
        ),
    ]
    driver: typing.Optional[DetectionLogDriverObjectResponseBody] = None
    id: str = pydantic.Field()
    """
    Unique Samsara ID (uuid) of the detection.
    """

    in_cab_alert_played: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="inCabAlertPlayed"),
        pydantic.Field(
            alias="inCabAlertPlayed", description="Whether an in cab alert played aloud in the cab. Always returned."
        ),
    ]
    safety_event: typing_extensions.Annotated[
        typing.List[DetectionLogSafetyEventObjectResponseBody],
        FieldMetadata(alias="safetyEvent"),
        pydantic.Field(
            alias="safetyEvent", description="Details on the associated safety event generated. Always returned."
        ),
    ]
    trigger_detection_label: typing_extensions.Annotated[
        DetectionLogDetectionObjectResponseBodyTriggerDetectionLabel,
        FieldMetadata(alias="triggerDetectionLabel"),
        pydantic.Field(
            alias="triggerDetectionLabel",
            description="The label associated with the detection. Always returned.  Valid values: `acceleration`, `braking`, `crash`, `drowsy`, `eatingDrinking`, `edgeRailroadCrossingViolation`, `followingDistance`, `forwardCollisionWarning`, `genericDistraction`, `harshTurn`, `heavySpeeding`, `laneDeparture`, `lightSpeeding`, `maxSpeed`, `mobileUsage`, `moderateSpeeding`, `noSeatbelt`, `obstructedCamera`, `passenger`, `policyViolationMask`, `ranRedLight`, `rearCollisionWarning`, `rollingStop`, `rolloverProtection`, `severeSpeeding`, `smoking`, `speeding`, `unsafeParking`, `vehicleInBlindSpotWarning`, `vulnerableRoadUserCollisionWarning`, `yawControl`",
        ),
    ]
    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime",
            description="Time the detection was updated in Samsara in UTC. RFC 3339 format. Always returned.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.ifta_create_ifta_detail_job_response_body import IftaCreateIftaDetailJobResponseBody
from ..types.ifta_get_ifta_detail_job_response_body import IftaGetIftaDetailJobResponseBody
from ..types.ifta_get_ifta_jurisdiction_reports_response_body import IftaGetIftaJurisdictionReportsResponseBody
from ..types.ifta_get_ifta_vehicle_reports_response_body import IftaGetIftaVehicleReportsResponseBody
from .raw_client import AsyncRawIftaClient, RawIftaClient
from .types.get_ifta_jurisdiction_reports_request_fuel_type import GetIftaJurisdictionReportsRequestFuelType
from .types.get_ifta_jurisdiction_reports_request_month import GetIftaJurisdictionReportsRequestMonth
from .types.get_ifta_jurisdiction_reports_request_quarter import GetIftaJurisdictionReportsRequestQuarter
from .types.get_ifta_vehicle_reports_request_fuel_type import GetIftaVehicleReportsRequestFuelType
from .types.get_ifta_vehicle_reports_request_month import GetIftaVehicleReportsRequestMonth
from .types.get_ifta_vehicle_reports_request_quarter import GetIftaVehicleReportsRequestQuarter

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class IftaClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawIftaClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawIftaClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawIftaClient
        """
        return self._raw_client

    def get_ifta_jurisdiction_reports(
        self,
        *,
        year: int,
        month: typing.Optional[GetIftaJurisdictionReportsRequestMonth] = None,
        quarter: typing.Optional[GetIftaJurisdictionReportsRequestQuarter] = None,
        jurisdictions: typing.Optional[str] = None,
        fuel_type: typing.Optional[GetIftaJurisdictionReportsRequestFuelType] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IftaGetIftaJurisdictionReportsResponseBody:
        """
        Get all jurisdiction IFTA reports for the requested time duration. Data is returned in your organization's defined timezone.

        **Note:** The most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        year : int
             The year of the requested IFTA report summary. Must be provided with a month or quarter param. Example: `year=2021`

        month : typing.Optional[GetIftaJurisdictionReportsRequestMonth]
             The month of the requested IFTA report summary. Can not be provided with the quarter param. Example: `month=January`  Valid values: `January`, `February`, `March`, `April`, `May`, `June`, `July`, `August`, `September`, `October`, `November`, `December`

        quarter : typing.Optional[GetIftaJurisdictionReportsRequestQuarter]
             The quarter of the requested IFTA report summary. Can not be provided with the month param. Q1: January, February, March. Q2: April, May, June. Q3: July, August, September. Q4: October, November, December. Example: `quarter=Q1`  Valid values: `Q1`, `Q2`, `Q3`, `Q4`

        jurisdictions : typing.Optional[str]
             A filter on the data based on this comma-separated list of jurisdictions. Example: `jurisdictions=GA`

        fuel_type : typing.Optional[GetIftaJurisdictionReportsRequestFuelType]
             A filter on the data based on this comma-separated list of IFTA fuel types. Example: `fuelType=Diesel`  Valid values: `Unspecified`, `A55`, `Biodiesel`, `CompressedNaturalGas`, `Diesel`, `E85`, `Electricity`, `Ethanol`, `Gasohol`, `Gasoline`, `Hydrogen`, `LiquifiedNaturalGas`, `M85`, `Methanol`, `Propane`, `Other`

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaGetIftaJurisdictionReportsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.ifta.get_ifta_jurisdiction_reports(
            year=1000000,
        )
        """
        _response = self._raw_client.get_ifta_jurisdiction_reports(
            year=year,
            month=month,
            quarter=quarter,
            jurisdictions=jurisdictions,
            fuel_type=fuel_type,
            vehicle_ids=vehicle_ids,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            request_options=request_options,
        )
        return _response.data

    def get_ifta_vehicle_reports(
        self,
        *,
        year: int,
        month: typing.Optional[GetIftaVehicleReportsRequestMonth] = None,
        quarter: typing.Optional[GetIftaVehicleReportsRequestQuarter] = None,
        jurisdictions: typing.Optional[str] = None,
        fuel_type: typing.Optional[GetIftaVehicleReportsRequestFuelType] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IftaGetIftaVehicleReportsResponseBody:
        """
        Get all vehicle IFTA reports for the requested time duration. Data is returned in your organization's defined timezone.

        **Note:** The most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        year : int
             The year of the requested IFTA report summary. Must be provided with a month or quarter param. Example: `year=2021`

        month : typing.Optional[GetIftaVehicleReportsRequestMonth]
             The month of the requested IFTA report summary. Can not be provided with the quarter param. Example: `month=January`  Valid values: `January`, `February`, `March`, `April`, `May`, `June`, `July`, `August`, `September`, `October`, `November`, `December`

        quarter : typing.Optional[GetIftaVehicleReportsRequestQuarter]
             The quarter of the requested IFTA report summary. Can not be provided with the month param. Q1: January, February, March. Q2: April, May, June. Q3: July, August, September. Q4: October, November, December. Example: `quarter=Q1`  Valid values: `Q1`, `Q2`, `Q3`, `Q4`

        jurisdictions : typing.Optional[str]
             A filter on the data based on this comma-separated list of jurisdictions. Example: `jurisdictions=GA`

        fuel_type : typing.Optional[GetIftaVehicleReportsRequestFuelType]
             A filter on the data based on this comma-separated list of IFTA fuel types. Example: `fuelType=Diesel`  Valid values: `Unspecified`, `A55`, `Biodiesel`, `CompressedNaturalGas`, `Diesel`, `E85`, `Electricity`, `Ethanol`, `Gasohol`, `Gasoline`, `Hydrogen`, `LiquifiedNaturalGas`, `M85`, `Methanol`, `Propane`, `Other`

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaGetIftaVehicleReportsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.ifta.get_ifta_vehicle_reports(
            year=1000000,
        )
        """
        _response = self._raw_client.get_ifta_vehicle_reports(
            year=year,
            month=month,
            quarter=quarter,
            jurisdictions=jurisdictions,
            fuel_type=fuel_type,
            vehicle_ids=vehicle_ids,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    def create_ifta_detail_job(
        self,
        *,
        end_hour: str,
        start_hour: str,
        vehicle_ids: typing.Optional[str] = OMIT,
        vehicle_parent_tag_ids: typing.Optional[str] = OMIT,
        vehicle_tag_ids: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IftaCreateIftaDetailJobResponseBody:
        """
        Create a job to generate csv files of IFTA mileage segments.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        end_hour : str
             An end time in RFC 3339 format. Hour precision and timezones are supported. Any minutes or seconds will be truncated down to the nearest hour. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. The maximum request duration is 1 month. Limit the number of vehicles to 1000 when requesting more than 24 hours of data. (Examples: 2019-06-13T19:00:00Z, 2019-06-13T19:00:00.000Z, OR 2015-09-15T14:00:00-04:00).

        start_hour : str
             A start time in RFC 3339 format. Hour precision and timezones are supported. Any minutes or seconds will be truncated down to the nearest hour. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. The maximum request duration is 1 month. Limit the number of vehicles to 1000 when requesting more than 24 hours of data. (Examples: 2019-06-13T19:00:00Z, 2019-06-13T19:00:00.000Z, OR 2015-09-15T14:00:00-04:00).

        vehicle_ids : typing.Optional[str]
            A filter on the data based on this comma-separated list of vehicle IDs and external IDs. The number of vehicles requested per job shouldn't exceed 5000. Example: `vehicleIds: '1234,5678,samsara.vin:1HGBH41JXMN109186'`

        vehicle_parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle parent tag IDs. The number of vehicles requested per job shouldn't exceed 5000. Example: `vehicleParentTagIds: '1234,5678'`

        vehicle_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle tag IDs. The number of vehicles requested per job shouldn't exceed 5000. Example: `vehicleTagIds: '1234,5678'`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaCreateIftaDetailJobResponseBody
            Created response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.ifta.create_ifta_detail_job(
            end_hour="2019-06-13T19:00:00Z",
            start_hour="2019-06-13T19:00:00Z",
        )
        """
        _response = self._raw_client.create_ifta_detail_job(
            end_hour=end_hour,
            start_hour=start_hour,
            vehicle_ids=vehicle_ids,
            vehicle_parent_tag_ids=vehicle_parent_tag_ids,
            vehicle_tag_ids=vehicle_tag_ids,
            request_options=request_options,
        )
        return _response.data

    def get_ifta_detail_job(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> IftaGetIftaDetailJobResponseBody:
        """
        Get information about an existing IFTA detail job.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the requested job.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaGetIftaDetailJobResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.ifta.get_ifta_detail_job(
            id="id",
        )
        """
        _response = self._raw_client.get_ifta_detail_job(id, request_options=request_options)
        return _response.data


class AsyncIftaClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawIftaClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawIftaClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawIftaClient
        """
        return self._raw_client

    async def get_ifta_jurisdiction_reports(
        self,
        *,
        year: int,
        month: typing.Optional[GetIftaJurisdictionReportsRequestMonth] = None,
        quarter: typing.Optional[GetIftaJurisdictionReportsRequestQuarter] = None,
        jurisdictions: typing.Optional[str] = None,
        fuel_type: typing.Optional[GetIftaJurisdictionReportsRequestFuelType] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IftaGetIftaJurisdictionReportsResponseBody:
        """
        Get all jurisdiction IFTA reports for the requested time duration. Data is returned in your organization's defined timezone.

        **Note:** The most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        year : int
             The year of the requested IFTA report summary. Must be provided with a month or quarter param. Example: `year=2021`

        month : typing.Optional[GetIftaJurisdictionReportsRequestMonth]
             The month of the requested IFTA report summary. Can not be provided with the quarter param. Example: `month=January`  Valid values: `January`, `February`, `March`, `April`, `May`, `June`, `July`, `August`, `September`, `October`, `November`, `December`

        quarter : typing.Optional[GetIftaJurisdictionReportsRequestQuarter]
             The quarter of the requested IFTA report summary. Can not be provided with the month param. Q1: January, February, March. Q2: April, May, June. Q3: July, August, September. Q4: October, November, December. Example: `quarter=Q1`  Valid values: `Q1`, `Q2`, `Q3`, `Q4`

        jurisdictions : typing.Optional[str]
             A filter on the data based on this comma-separated list of jurisdictions. Example: `jurisdictions=GA`

        fuel_type : typing.Optional[GetIftaJurisdictionReportsRequestFuelType]
             A filter on the data based on this comma-separated list of IFTA fuel types. Example: `fuelType=Diesel`  Valid values: `Unspecified`, `A55`, `Biodiesel`, `CompressedNaturalGas`, `Diesel`, `E85`, `Electricity`, `Ethanol`, `Gasohol`, `Gasoline`, `Hydrogen`, `LiquifiedNaturalGas`, `M85`, `Methanol`, `Propane`, `Other`

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaGetIftaJurisdictionReportsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.ifta.get_ifta_jurisdiction_reports(
                year=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_ifta_jurisdiction_reports(
            year=year,
            month=month,
            quarter=quarter,
            jurisdictions=jurisdictions,
            fuel_type=fuel_type,
            vehicle_ids=vehicle_ids,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            request_options=request_options,
        )
        return _response.data

    async def get_ifta_vehicle_reports(
        self,
        *,
        year: int,
        month: typing.Optional[GetIftaVehicleReportsRequestMonth] = None,
        quarter: typing.Optional[GetIftaVehicleReportsRequestQuarter] = None,
        jurisdictions: typing.Optional[str] = None,
        fuel_type: typing.Optional[GetIftaVehicleReportsRequestFuelType] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IftaGetIftaVehicleReportsResponseBody:
        """
        Get all vehicle IFTA reports for the requested time duration. Data is returned in your organization's defined timezone.

        **Note:** The most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        year : int
             The year of the requested IFTA report summary. Must be provided with a month or quarter param. Example: `year=2021`

        month : typing.Optional[GetIftaVehicleReportsRequestMonth]
             The month of the requested IFTA report summary. Can not be provided with the quarter param. Example: `month=January`  Valid values: `January`, `February`, `March`, `April`, `May`, `June`, `July`, `August`, `September`, `October`, `November`, `December`

        quarter : typing.Optional[GetIftaVehicleReportsRequestQuarter]
             The quarter of the requested IFTA report summary. Can not be provided with the month param. Q1: January, February, March. Q2: April, May, June. Q3: July, August, September. Q4: October, November, December. Example: `quarter=Q1`  Valid values: `Q1`, `Q2`, `Q3`, `Q4`

        jurisdictions : typing.Optional[str]
             A filter on the data based on this comma-separated list of jurisdictions. Example: `jurisdictions=GA`

        fuel_type : typing.Optional[GetIftaVehicleReportsRequestFuelType]
             A filter on the data based on this comma-separated list of IFTA fuel types. Example: `fuelType=Diesel`  Valid values: `Unspecified`, `A55`, `Biodiesel`, `CompressedNaturalGas`, `Diesel`, `E85`, `Electricity`, `Ethanol`, `Gasohol`, `Gasoline`, `Hydrogen`, `LiquifiedNaturalGas`, `M85`, `Methanol`, `Propane`, `Other`

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaGetIftaVehicleReportsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.ifta.get_ifta_vehicle_reports(
                year=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_ifta_vehicle_reports(
            year=year,
            month=month,
            quarter=quarter,
            jurisdictions=jurisdictions,
            fuel_type=fuel_type,
            vehicle_ids=vehicle_ids,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    async def create_ifta_detail_job(
        self,
        *,
        end_hour: str,
        start_hour: str,
        vehicle_ids: typing.Optional[str] = OMIT,
        vehicle_parent_tag_ids: typing.Optional[str] = OMIT,
        vehicle_tag_ids: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> IftaCreateIftaDetailJobResponseBody:
        """
        Create a job to generate csv files of IFTA mileage segments.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        end_hour : str
             An end time in RFC 3339 format. Hour precision and timezones are supported. Any minutes or seconds will be truncated down to the nearest hour. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. The maximum request duration is 1 month. Limit the number of vehicles to 1000 when requesting more than 24 hours of data. (Examples: 2019-06-13T19:00:00Z, 2019-06-13T19:00:00.000Z, OR 2015-09-15T14:00:00-04:00).

        start_hour : str
             A start time in RFC 3339 format. Hour precision and timezones are supported. Any minutes or seconds will be truncated down to the nearest hour. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. The maximum request duration is 1 month. Limit the number of vehicles to 1000 when requesting more than 24 hours of data. (Examples: 2019-06-13T19:00:00Z, 2019-06-13T19:00:00.000Z, OR 2015-09-15T14:00:00-04:00).

        vehicle_ids : typing.Optional[str]
            A filter on the data based on this comma-separated list of vehicle IDs and external IDs. The number of vehicles requested per job shouldn't exceed 5000. Example: `vehicleIds: '1234,5678,samsara.vin:1HGBH41JXMN109186'`

        vehicle_parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle parent tag IDs. The number of vehicles requested per job shouldn't exceed 5000. Example: `vehicleParentTagIds: '1234,5678'`

        vehicle_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle tag IDs. The number of vehicles requested per job shouldn't exceed 5000. Example: `vehicleTagIds: '1234,5678'`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaCreateIftaDetailJobResponseBody
            Created response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.ifta.create_ifta_detail_job(
                end_hour="2019-06-13T19:00:00Z",
                start_hour="2019-06-13T19:00:00Z",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_ifta_detail_job(
            end_hour=end_hour,
            start_hour=start_hour,
            vehicle_ids=vehicle_ids,
            vehicle_parent_tag_ids=vehicle_parent_tag_ids,
            vehicle_tag_ids=vehicle_tag_ids,
            request_options=request_options,
        )
        return _response.data

    async def get_ifta_detail_job(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> IftaGetIftaDetailJobResponseBody:
        """
        Get information about an existing IFTA detail job.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read IFTA (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            ID of the requested job.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        IftaGetIftaDetailJobResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.ifta.get_ifta_detail_job(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_ifta_detail_job(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .v_1_machine_history_response_vibrations import V1MachineHistoryResponseVibrations


class V1MachineHistoryResponseMachines(UniversalBaseModel):
    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Machine ID
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Machine name
    """

    vibrations: typing.Optional[typing.List[V1MachineHistoryResponseVibrations]] = pydantic.Field(default=None)
    """
    List of vibration datapoints, with timestamp and vibration measurement for x/y/z axis in mm/s
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .base_route_response_object_response_body import BaseRouteResponseObjectResponseBody
from .route_changes_response_body import RouteChangesResponseBody
from .route_feed_object_response_body_operation import RouteFeedObjectResponseBodyOperation
from .route_feed_object_response_body_source import RouteFeedObjectResponseBodySource
from .route_feed_object_response_body_type import RouteFeedObjectResponseBodyType


class RouteFeedObjectResponseBody(UniversalBaseModel):
    """
    Route feed object.
    """

    changes: RouteChangesResponseBody
    operation: typing.Optional[RouteFeedObjectResponseBodyOperation] = pydantic.Field(default=None)
    """
    The operation that was performed as part of this route update.  Valid values: `stop scheduled`, `stop en route`, `stop skipped`, `stop arrived`, `stop departed`, `stop ETA updated`, `stop arrival time updated`, `stop completion time updated`, `stop order changed`, `stop arrival prevented`
    """

    route: BaseRouteResponseObjectResponseBody
    source: RouteFeedObjectResponseBodySource = pydantic.Field()
    """
    The source of this route update. Updates that are triggered by time or by the route being completed are 'automatic'.  Valid values: `automatic`, `driver`, `admin`
    """

    time: dt.datetime = pydantic.Field()
    """
    The timestamp of the route in RFC 3339 format.
    """

    type: RouteFeedObjectResponseBodyType = pydantic.Field()
    """
    The type of route update. The route tracking updates occur as a route is completed and stops transition from one state to another. Currently only Route Tracking updates are supported, but this will change in the future when additional types are added.  Valid values: `route tracking`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FieldTypesObjectResponseBodyFieldType = typing.Union[
    typing.Literal[
        "photo", "string", "number", "multipleChoice", "signature", "dateTime", "scannedDocument", "barcode"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_conditional_action_object_response_body import FormsConditionalActionObjectResponseBody
from .forms_field_definition_object_response_body_allowed_asset_types_item import (
    FormsFieldDefinitionObjectResponseBodyAllowedAssetTypesItem,
)
from .forms_field_definition_object_response_body_allowed_date_time_value_type import (
    FormsFieldDefinitionObjectResponseBodyAllowedDateTimeValueType,
)
from .forms_field_definition_object_response_body_type import FormsFieldDefinitionObjectResponseBodyType
from .forms_select_option_object_response_body import FormsSelectOptionObjectResponseBody
from .forms_table_field_definition_object_response_body import FormsTableFieldDefinitionObjectResponseBody


class FormsFieldDefinitionObjectResponseBody(UniversalBaseModel):
    """
    Forms field definition object.
    """

    allow_manual_entry: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="allowManualEntry"),
        pydantic.Field(
            alias="allowManualEntry",
            description="Indicates whether the field allows manual entry of a person. Only present for person fields.",
        ),
    ] = None
    allowed_asset_types: typing_extensions.Annotated[
        typing.Optional[typing.List[FormsFieldDefinitionObjectResponseBodyAllowedAssetTypesItem]],
        FieldMetadata(alias="allowedAssetTypes"),
        pydantic.Field(
            alias="allowedAssetTypes",
            description="List of allowed asset types that can be selected for this field. Only present for asset fields.",
        ),
    ] = None
    allowed_date_time_value_type: typing_extensions.Annotated[
        typing.Optional[FormsFieldDefinitionObjectResponseBodyAllowedDateTimeValueType],
        FieldMetadata(alias="allowedDateTimeValueType"),
        pydantic.Field(
            alias="allowedDateTimeValueType",
            description="Type of date/time entry allowed for this question. Only present for datetime fields.  Valid values: `datetime`, `date`, `time`",
        ),
    ] = None
    autofill_from_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="autofillFromId"),
        pydantic.Field(
            alias="autofillFromId",
            description="Identifier of the field that will optionally autofill the current field. Only present for fields that have been configured to be autofilled by a source media field.",
        ),
    ] = None
    columns: typing.Optional[typing.List[FormsTableFieldDefinitionObjectResponseBody]] = pydantic.Field(default=None)
    """
    List of columns in the table field.
    """

    conditional_actions: typing_extensions.Annotated[
        typing.Optional[typing.List[FormsConditionalActionObjectResponseBody]],
        FieldMetadata(alias="conditionalActions"),
        pydantic.Field(alias="conditionalActions", description="List of conditional actions."),
    ] = None
    filter_by_role_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="filterByRoleIds"),
        pydantic.Field(
            alias="filterByRoleIds",
            description="List of role IDs to filter org users by, representing which roles are selectable people for this field. Only present for person fields.",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    Identifier of the field.
    """

    include_drivers: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="includeDrivers"),
        pydantic.Field(
            alias="includeDrivers",
            description="Indicates whether the field includes drivers as selectable people. Only present for person fields.",
        ),
    ] = None
    include_users: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="includeUsers"),
        pydantic.Field(
            alias="includeUsers",
            description="Indicates whether the field includes users as selectable people. Only present for person fields.",
        ),
    ] = None
    is_autofill_source: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isAutofillSource"),
        pydantic.Field(
            alias="isAutofillSource",
            description="Indicates whether the current field is enabled to autofill other fields. Only present for media fields that have autofill enabled.",
        ),
    ] = None
    is_required: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isRequired"),
        pydantic.Field(
            alias="isRequired", description="Indicates whether the form field is required to be filled out by the user."
        ),
    ]
    label: str = pydantic.Field()
    """
    Label of the field.
    """

    num_decimal_places: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="numDecimalPlaces"),
        pydantic.Field(
            alias="numDecimalPlaces", description="Number of decimal places allowed. Only present for number fields."
        ),
    ] = None
    options: typing.Optional[typing.List[FormsSelectOptionObjectResponseBody]] = pydantic.Field(default=None)
    """
    List of select options for check boxes or multiple choice fields.
    """

    question_weight: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="questionWeight"),
        pydantic.Field(
            alias="questionWeight",
            description="The maximum possible score weight for this field. For multiple choice fields, this number is the highest option score weight of the given options. For check boxes fields, this number is the sum of the score weights for all scored options. Only present for multiple choice or check boxes fields that have scoring.",
        ),
    ] = None
    type: FormsFieldDefinitionObjectResponseBodyType = pydantic.Field()
    """
    Type of the field.  Valid values: `number`, `text`, `multiple_choice`, `check_boxes`, `media`, `datetime`, `signature`, `asset`, `person`, `geofence`, `instruction`, `media_instruction`, `table`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .behavior_response_body import BehaviorResponseBody
from .coaching_sessions_response_response_body_coaching_type import CoachingSessionsResponseResponseBodyCoachingType
from .coaching_sessions_response_response_body_session_status import CoachingSessionsResponseResponseBodySessionStatus
from .driver_with_external_id_object_response_body import DriverWithExternalIdObjectResponseBody


class CoachingSessionsResponseResponseBody(UniversalBaseModel):
    """
    List of coaching sessions objects.
    """

    assigned_coach_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assignedCoachId"),
        pydantic.Field(
            alias="assignedCoachId",
            description="Unique user ID for a coaching session. Returned when a coaching session status is “incomplete”.",
        ),
    ] = None
    behaviors: typing.List[BehaviorResponseBody] = pydantic.Field()
    """
    Object references for the behaviors within the session.
    """

    coaching_type: typing_extensions.Annotated[
        CoachingSessionsResponseResponseBodyCoachingType,
        FieldMetadata(alias="coachingType"),
        pydantic.Field(
            alias="coachingType",
            description="Coaching type for the coaching session.  Valid values: `fullySharedWithManager`, `selfCoaching`, `unknown`, `unshared`, `withManager`",
        ),
    ]
    completed_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="completedAtTime"),
        pydantic.Field(
            alias="completedAtTime",
            description="Time coaching session is completed in UTC. Returned when a coaching session status is “completed”.",
        ),
    ] = None
    completed_coach_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="completedCoachId"),
        pydantic.Field(
            alias="completedCoachId",
            description="Unique user ID for a completed coaching session. Returned when a coaching session status is “completed”.",
        ),
    ] = None
    driver: DriverWithExternalIdObjectResponseBody
    due_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="dueAtTime"),
        pydantic.Field(alias="dueAtTime", description="Time coaching session is due in UTC."),
    ]
    id: str = pydantic.Field()
    """
    Unique ID for the coaching session.
    """

    session_note: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="sessionNote"),
        pydantic.Field(
            alias="sessionNote", description="Associated note for the coaching session. Returned when present."
        ),
    ] = None
    session_status: typing_extensions.Annotated[
        CoachingSessionsResponseResponseBodySessionStatus,
        FieldMetadata(alias="sessionStatus"),
        pydantic.Field(
            alias="sessionStatus",
            description="Status for the coaching session.  Valid values: `unknown`, `upcoming`, `completed`, `deleted`",
        ),
    ]
    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(alias="updatedAtTime", description="Time coaching session was updated in UTC."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .tiny_asset_object_response_body_asset_type import TinyAssetObjectResponseBodyAssetType


class TinyAssetObjectResponseBody(UniversalBaseModel):
    """
    Vehicle, trailer or other equipment to be tracked.
    """

    asset_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="assetId"), pydantic.Field(alias="assetId", description="ID of the asset.")
    ]
    asset_type: typing_extensions.Annotated[
        TinyAssetObjectResponseBodyAssetType,
        FieldMetadata(alias="assetType"),
        pydantic.Field(
            alias="assetType",
            description="The operational context in which the asset interacts with the Samsara system. Examples: Vehicle (eg: truck, bus...), Trailer (eg: dry van, reefer, flatbed...), Powered Equipment (eg: dozer, crane...), Unpowered Equipment (eg: container, dumpster...), or Uncategorized.  Valid values: `uncategorized`, `trailer`, `equipment`, `unpowered`, `vehicle`",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VehicleStatsFaultCodesValueObdiiConfirmedDtcs(UniversalBaseModel):
    """
    Passenger vehicle DTC information
    """

    dtc_description: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dtcDescription"),
        pydantic.Field(alias="dtcDescription", description="The DTC description, if available."),
    ] = None
    dtc_id: typing_extensions.Annotated[
        int, FieldMetadata(alias="dtcId"), pydantic.Field(alias="dtcId", description="The DTC identifier.")
    ]
    dtc_short_code: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dtcShortCode"),
        pydantic.Field(alias="dtcShortCode", description="The DTC short code, if available."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1HosAuthenticationLogsResponseAuthenticationLogs(UniversalBaseModel):
    action_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="actionType"),
        pydantic.Field(alias="actionType", description="The log type - one of 'signin' or 'signout'"),
    ] = None
    address: typing.Optional[str] = pydantic.Field(default=None)
    """
    DEPRECATED: THIS FIELD MAY NOT BE POPULATED
    """

    address_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="addressName"),
        pydantic.Field(alias="addressName", description="DEPRECATED: THIS FIELD MAY NOT BE POPULATED"),
    ] = None
    city: typing.Optional[str] = pydantic.Field(default=None)
    """
    DEPRECATED: THIS FIELD MAY NOT BE POPULATED
    """

    happened_at_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="happenedAtMs"),
        pydantic.Field(
            alias="happenedAtMs", description="The time at which the event was recorded in UNIX milliseconds."
        ),
    ] = None
    state: typing.Optional[str] = pydantic.Field(default=None)
    """
    DEPRECATED: THIS FIELD MAY NOT BE POPULATED
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_temperature_response_sensors import V1TemperatureResponseSensors


class V1TemperatureResponse(UniversalBaseModel):
    """
    Contains the current temperatures of a sensor.
    """

    group_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="groupId"), pydantic.Field(alias="groupId", description="Deprecated.")
    ] = None
    sensors: typing.Optional[typing.List[V1TemperatureResponseSensors]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .goa_pagination_response_response_body import GoaPaginationResponseResponseBody
from .idling_report_event_response_body import IdlingReportEventResponseBody


class IdlingReportsGetVehicleIdlingReportsResponseBody(UniversalBaseModel):
    data: typing.List[IdlingReportEventResponseBody] = pydantic.Field()
    """
    Multiple idling events.
    """

    pagination: GoaPaginationResponseResponseBody
    request_end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="requestEndTime"),
        pydantic.Field(alias="requestEndTime", description="The requested end time in RFC 3339 format."),
    ]
    request_start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="requestStartTime"),
        pydantic.Field(alias="requestStartTime", description="The requested start time in RFC 3339 format."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .v_1_vision_cameras_response_item import V1VisionCamerasResponseItem

V1VisionCamerasResponse = typing.List[V1VisionCamerasResponseItem]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .carrier_proposed_assignment_accepted_time import CarrierProposedAssignmentAcceptedTime
from .carrier_proposed_assignment_active_time import CarrierProposedAssignmentActiveTime
from .carrier_proposed_assignment_driver import CarrierProposedAssignmentDriver
from .carrier_proposed_assignment_first_seen_time import CarrierProposedAssignmentFirstSeenTime
from .carrier_proposed_assignment_id import CarrierProposedAssignmentId
from .carrier_proposed_assignment_rejected_time import CarrierProposedAssignmentRejectedTime
from .carrier_proposed_assignment_shipping_docs import CarrierProposedAssignmentShippingDocs
from .carrier_proposed_assignment_trailers import CarrierProposedAssignmentTrailers
from .carrier_proposed_assignment_vehicle import CarrierProposedAssignmentVehicle


class CarrierProposedAssignment(UniversalBaseModel):
    """
    A carrier proposed assignment object
    """

    accepted_time: typing_extensions.Annotated[
        typing.Optional[CarrierProposedAssignmentAcceptedTime],
        FieldMetadata(alias="acceptedTime"),
        pydantic.Field(alias="acceptedTime"),
    ] = None
    active_time: typing_extensions.Annotated[
        CarrierProposedAssignmentActiveTime, FieldMetadata(alias="activeTime"), pydantic.Field(alias="activeTime")
    ]
    driver: typing.Optional[CarrierProposedAssignmentDriver] = None
    first_seen_time: typing_extensions.Annotated[
        typing.Optional[CarrierProposedAssignmentFirstSeenTime],
        FieldMetadata(alias="firstSeenTime"),
        pydantic.Field(alias="firstSeenTime"),
    ] = None
    id: CarrierProposedAssignmentId
    rejected_time: typing_extensions.Annotated[
        typing.Optional[CarrierProposedAssignmentRejectedTime],
        FieldMetadata(alias="rejectedTime"),
        pydantic.Field(alias="rejectedTime"),
    ] = None
    shipping_docs: typing_extensions.Annotated[
        typing.Optional[CarrierProposedAssignmentShippingDocs],
        FieldMetadata(alias="shippingDocs"),
        pydantic.Field(alias="shippingDocs"),
    ] = None
    trailers: typing.Optional[CarrierProposedAssignmentTrailers] = None
    vehicle: typing.Optional[CarrierProposedAssignmentVehicle] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FuelPurchasePostFuelPurchaseRequestBodyIftaFuelType = typing.Union[
    typing.Literal[
        "Unspecified",
        "A55",
        "Biodiesel",
        "CompressedNaturalGas",
        "Diesel",
        "E85",
        "Electricity",
        "Ethanol",
        "Gasohol",
        "Gasoline",
        "Hydrogen",
        "LiquifiedNaturalGas",
        "M85",
        "Methanol",
        "Propane",
        "Other",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_app_notification_object_request_body import DriverAppNotificationObjectRequestBody
from .recipient_object_request_body import RecipientObjectRequestBody
from .webhook_params_object_request_body import WebhookParamsObjectRequestBody


class ActionParamsObjectRequestBody(UniversalBaseModel):
    """
    The action type specific details. Set webhookIds for Slack or Webhook actions. Set recipients for Notifications. Set driverAppNotification for Driver App Push. Other action types don't need to set a param.
    """

    driver_app_notification: typing_extensions.Annotated[
        typing.Optional[DriverAppNotificationObjectRequestBody],
        FieldMetadata(alias="driverAppNotification"),
        pydantic.Field(alias="driverAppNotification"),
    ] = None
    recipients: typing.Optional[typing.List[RecipientObjectRequestBody]] = pydantic.Field(default=None)
    """
    Recipient of the action.
    """

    webhooks: typing.Optional[WebhookParamsObjectRequestBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

UploadedMediaObjectResponseBodyMediaType = typing.Union[
    typing.Literal["image", "videoHighRes", "videoLowRes", "hyperlapse"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HosEldEventLocationObjectResponseBody(UniversalBaseModel):
    city: typing.Optional[str] = pydantic.Field(default=None)
    """
    The best effort city for the latitude and longitude.
    """

    eld_location: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="eldLocation"),
        pydantic.Field(
            alias="eldLocation",
            description="Relative location to the city, village, or town with population of 5,000 or greater.",
        ),
    ] = None
    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    The latitude of the location.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    The longitude of the location.
    """

    state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The best effort state for the latitude and longitude.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1VisionStepResultsItemDistance(UniversalBaseModel):
    distance_found: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="distanceFound"),
        pydantic.Field(alias="distanceFound", description="The distance found between the start and end references"),
    ] = None
    end_step_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endStepName"),
        pydantic.Field(
            alias="endStepName",
            description="The name of the second reference step that we're checking the distances between",
        ),
    ] = None
    enforce_offset_angle_range: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="enforceOffsetAngleRange"),
        pydantic.Field(alias="enforceOffsetAngleRange", description="Whether an offset angle range is enforced"),
    ] = None
    max_distance: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="maxDistance"),
        pydantic.Field(alias="maxDistance", description="The maximum allowed distance threshold"),
    ] = None
    max_offset_angle: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="maxOffsetAngle"),
        pydantic.Field(
            alias="maxOffsetAngle",
            description="The maximum angle allowance (in degrees) if enforceOffsetAngleRange is true",
        ),
    ] = None
    min_distance: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="minDistance"),
        pydantic.Field(alias="minDistance", description="The minumum allowed distance threshold"),
    ] = None
    min_offset_angle: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="minOffsetAngle"),
        pydantic.Field(
            alias="minOffsetAngle",
            description="The minimum angle allowance (in degrees) if enforceOffsetAngleRange is true",
        ),
    ] = None
    offset_angle_found: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="offsetAngleFound"),
        pydantic.Field(
            alias="offsetAngleFound",
            description="The counter-clockwise angle (in degrees) found between the horizontal axis of the start reference step and the last",
        ),
    ] = None
    start_step_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="startStepName"),
        pydantic.Field(
            alias="startStepName",
            description="The name of the first reference step that we're checking the distances between",
        ),
    ] = None
    unit: typing.Optional[str] = pydantic.Field(default=None)
    """
    The measurement unit of the distance found and the min and max distance threshold
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

IftaDetailJobResponseBodyJobStatus = typing.Union[
    typing.Literal["Requested", "Processing", "Completed", "Failed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class Dvir2AssignedToPolymorphicUserObjectResponseBody(UniversalBaseModel):
    """
    The user or driver assigned to complete the DVIR.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the user or driver assigned to the DVIR.
    """

    polymorphic_user_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="polymorphicUserId"),
        pydantic.Field(
            alias="polymorphicUserId", description="The polymorphic user ID that identifies the user or driver."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
